﻿/*
	Archimi IDTOOL v 1.2 (Corona)
	By Michel L. (michel@archimi.com)
	www.archimi.com
	

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
	
	
This script is used to manage objects ID interactively.
	- It allows to see the IDs.
	- To modify their IDs.
	- To randomly modify their IDs.
	- To select the objects by their IDs.
	- To select the objects by their materials.
		etc.
This without having to close the script during your work.

And especially, for CORONA users, this script allows you to update the VFB when you change the IDs, without having to restart the interactive rendering !!

You can find the script in “Archimi” category (Customize/Customize User Interface/Archimi)

Change log:

V 1.2
selects only visible objects
	
v 1.1 
fix some bugs

*/

macroScript Archimi_IDTool
	category:"Archimi"
	toolTip:"Archimi IDTool"

(	
	local version = "v1.2"
	
	try destroyDialog ::ArchimiIDTool catch()
	
	persistent global ArchimiIDToolPos
	persistent global ArchimiIDAutoUpdate
	persistent global ArchimiIDRandomMin
	persistent global ArchimiIDRandomMax
	if ArchimiIDToolPos == undefined do ArchimiIDToolPos = [400,400]
	if ArchimiIDAutoUpdate == undefined do ArchimiIDAutoUpdate = false
	if ArchimiIDRandomMin == undefined do ArchimiIDRandomMin = 0
	if ArchimiIDRandomMax == undefined do ArchimiIDRandomMax = 10
		
rollout ArchimiIDTool ("Archimi IDTool " + version) width:200
(
	
	Group "Assign Object GBuffer ID" 
		(
			spinner spn_IDValue "ID: " range:[0,65535,0] width:90 height:24 type:#integer scale:1 align:#center pos:[52,32]
			checkbox chk_autoUpdateID "Auto Assign" checked:ArchimiIDAutoUpdate
			button btn_AssignID "Assign ID" width:180 height:30 
		)
	
	Group "Material"
		(
			edittext edt_matname ""  height:17 width:120 align:#left enabled:true readOnly:true across:2
			button btn_SelSameMat "Select" align:#right width:50 height:17 
		)
		
	Group "Object GBuffer ID in project" 
		(
			multiListBox mlb_IDList "" height:10  width:180
			--checkbox chk_FilterMatOk "Use material as filter" align:#right checked:false
		)
		
	Group "Random ID"
		(
			spinner spn_RandomMin "min :" range:[0,65534,ArchimiIDRandomMin] width:90  type:#integer align:#left 
			button btn_AssignRandomID "Random ID" height:30 align:#right offset:[0,-20]
			spinner spn_RandomMax "max:" range:[0,65535,ArchimiIDRandomMax] width:90 type:#integer align:#left  offset:[0,-20] 
		)
		
		
			hyperlink ArchimiLink "www.archimi.com" address:"http://www.archimi.com/ressources/" align:#center
	
		on ArchimiIDTool open do --********************************************************************************************************************
		(
			ArchimiIDTool.IDUpdateList()
			ArchimiIDTool.IDUpdateSel()
			callbacks.addScript #selectionSetChanged "ArchimiIDTool.IDUpdateSel()" id:#ARCHIMIIDToolDialog
			callbacks.addScript #nodeCreated "ArchimiIDTool.IDUpdateList()" id:#ARCHIMIIDToolDialog
			callbacks.addScript #nodePostDelete "ArchimiIDTool.IDUpdateList()" id:#ARCHIMIIDToolDialog
			callbacks.addScript #nodePostMaterial "ArchimiIDTool.IDUpdateSel()" id:#ARCHIMIIDToolDialog
			callbacks.addScript #nodeFreeze "ArchimiIDTool.IDUpdateList()" id:#ARCHIMIIDToolDialog
			callbacks.addScript #nodeUnfreeze "ArchimiIDTool.IDUpdateList()" id:#ARCHIMIIDToolDialog
			
			btn_AssignID.enabled = not ArchimiIDAutoUpdate
			
		)
		
	on ArchimiIDTool close do --********************************************************************************************************************
		(
			callbacks.removescripts id:#ARCHIMIIDToolDialog
			ArchimiIDToolPos = getdialogpos ArchimiIDTool
			ArchimiIDAutoUpdate = chk_autoUpdateID.state
			ArchimiIDRandomMin = spn_RandomMin.value
			ArchimiIDRandomMax = spn_RandomMax.value
		)	
		
	-- Select ID in list
	on mlb_IDList selectionEnd do --********************************************************************************************************************
		(
			undo "ArchimiIDTool" on
			(
				local SelectObjIDArray = #()
				for i in (mlb_IDList.selection as array) do
				(
					for o in objects where (o.gbufferchannel == mlb_IDList.items[i] as integer) and (o.isFrozen == false) and (o.isHiddenInVpt == false) do append SelectObjIDArray o
				)
				select SelectObjIDArray
			)
		)
		
	--	active auto Update
	on chk_autoUpdateID changed theState do --*****************************************************************************************************
		( 
			undo "ArchimiIDTool" on
			(
				if theState == true then
					btn_AssignID.enabled = false
				else
					btn_AssignID.enabled = true
			)
		)
	
	-- ID to assign
	on spn_IDValue changed valeur arrows do --***************************************************************************************************************
	(
		undo "ArchimiIDTool" on
		(		
			(if not (arrows== true and chk_autoUpdateID.state == false) and selection.count != 0 then 
				(
					for i in selection do i.gbufferchannel = valeur
					completeRedraw()
					if CoronaRenderer.CoronaFp.getRenderType () > 1 then CoronaRenderer.CoronaFp.reparseInteractive () -- Redraw the VFB Corona
					
					mlb_IDList.items = (ArchimiIDTool.getObjectsID false)
					ArchimiIDTool.IDUpdateSel()
				)
		)
		)
	)
	
	-- Button active if Auto Update off
	on btn_AssignID pressed do --********************************************************************************************************************
		(
			undo "ArchimiIDTool" on
			(
			if selection.count != 0 then
				(
					for i in selection do i.gbufferchannel = spn_IDValue.value
					completeRedraw()
					if CoronaRenderer.CoronaFp.getRenderType () > 1 then CoronaRenderer.CoronaFp.reparseInteractive () -- Redraw the VFB Corona
					
			)	
			mlb_IDList.items = (ArchimiIDTool.getObjectsID false)
			ArchimiIDTool.IDUpdateSel()
		)
	)
	
	on btn_SelSameMat pressed do --********************************************************************************************************************
		(
			ArchimiIDTool.SelectSameObjMat()
		)
		
	on btn_AssignRandomID pressed do --********************************************************************************************************************
		(
			undo "ArchimiIDTool" on 
			(
				if selection.count != 0 then
				(
					for i in selection do
					(
						i.gbufferchannel = random spn_RandomMin.value spn_RandomMax.value
					)
				mlb_IDList.items = (ArchimiIDTool.getObjectsID false)
			ArchimiIDTool.IDUpdateSel()
					completeRedraw()
					if CoronaRenderer.CoronaFp.getRenderType () > 1 then CoronaRenderer.CoronaFp.reparseInteractive () -- Redraw the VFB Corona
					
				)
			)
		)
		
	fn IDUpdateList = --********************************************************************************************************************
	(
		mlb_IDList.items = (ArchimiIDTool.getObjectsID false)
	)
	
	fn IDUpdateSel = --********************************************************************************************************************
		(

			if selection.count != 0 then -- selection pleine
 			( 
				local ListObjectID = ArchimiIDTool.getObjectsID true
				local MultiMat =""
				btn_SelSameMat.enabled = true
				btn_AssignRandomID.enabled = true
				
				--Surlignage dans la liste déroulante 
				local getListboxPosition = #()
				if mlb_IDList.items.count != 0 then
				(
					for i = 1 to mlb_IDList.items.count do
					(
						for o in ListObjectID do
						(
							if mlb_IDList.items[i] == o then
							(
								append getListboxPosition i
							)
						)
					)
				)
				mlb_IDList.selection = getListboxPosition

 				if selection.count == 1 then  -- si un seul objet selectionné
				(
					ArchimiIDTool.spn_IDValue.value = (ListObjectID[1] as integer)
					if selection[1].material != undefined then edt_matname.text = selection[1].material.name else edt_matname.text = "None"
 					
				)
					
				else  -- si plusieurs objets selectionnés
 					(ArchimiIDTool.spn_IDValue.Indeterminate = true
 						
						if ListObjectID.count == 1 then ArchimiIDTool.spn_IDValue.value = (ListObjectID[1] as integer)
					
						-- look for multi material						
						
						if selection[1].material != undefined then MultiMat = selection[1].material.name else MultiMat = "None"
						for o in selection do 
						(
						
							if classof o != Dummy then (
								
							if o.material != undefined then 
								(if MultiMat != o.material.name then MultiMat = "*** Multi material ***")
							else
								(if MultiMat != "None" then MultiMat = "*** Multi material ***")
							)
													
						)
							
						edt_matname.text = MultiMat
						if MultiMat == "*** Multi material ***" then btn_SelSameMat.enabled = false
	
					)
 				)
 			else
 			( --selection vide
 				ArchimiIDTool.spn_IDValue.Indeterminate = true
				mlb_IDList.selection = 0
				edt_matname.text = ""
				btn_SelSameMat.enabled = false
				btn_AssignRandomID.enabled = false
 				)
		)
		
	fn getObjectsID sel = --********************************************************************************************************************
		(
		local GBufferEnsemble = #()
		for o in (if sel == false then objects else selection) where (o.gbufferchannel != -1 and o.isFrozen == false) do
		--for o in (if sel == false then objects else selection) where (o.gbufferchannel != -1 and o.isFrozen == false and o.isHiddenInVpt == false) do
		(
			if (findItem GBufferEnsemble (o.gbufferchannel as string)) == 0 then append GBufferEnsemble (o.gbufferchannel as string)
		)	
		if GBufferEnsemble.count > 1 then
		(
			fn sortEnsemble p1 p2 =
			(
				local Chkvalue = (p2 as integer) - (p1 as integer)
				case of
				(
					(Chkvalue < 0.): 1
					(Chkvalue > 0.): -1
					default: 0
				)
			)
			qsort GBufferEnsemble sortEnsemble
		)
		return GBufferEnsemble
		)
		
	fn SelectSameObjMat = --********************************************************************************************************************
	(
		local SelectObjMat = #()
		local mat = edt_matname.text
		if mat == "None" then
		(
			for o in objects where (o.material == undefined and o.isFrozen == false and o.isHiddenInVpt == false) do append SelectObjMat o
		)
		else (
		for o in objects where (o.material != undefined and o.isFrozen == false and o.isHiddenInVpt == false) do 
			(
				if o.material.name== mat then append SelectObjMat o
				)
			)		
	select SelectObjMat
	)
		
)
createDialog ArchimiIDTool pos:ArchimiIDToolPos
)